@@ -6,29 +6,34 @@
 
 #![doc(html_root_url = "https://doc.rust-lang.org/nightly/")]
 
+#![feature(in_band_lifetimes)]
 #![feature(nll)]
-#![feature(rustc_diagnostic_macros)]
 
 #![recursion_limit="256"]
 
-#![deny(rust_2018_idioms)]
-
 #[macro_use]
 extern crate rustc;
+#[macro_use]
+extern crate log;
+#[macro_use]
+extern crate syntax;
 
 use rustc::ty::query::Providers;
 
-mod diagnostics;
+pub mod error_codes;
 
 pub mod ast_validation;
-pub mod rvalue_promotion;
 pub mod hir_stats;
 pub mod layout_test;
 pub mod loops;
-
-__build_diagnostic_array! { librustc_passes, DIAGNOSTICS }
+pub mod dead;
+pub mod entry;
+mod liveness;
+mod intrinsicck;
 
 pub fn provide(providers: &mut Providers<'_>) {
-    rvalue_promotion::provide(providers);
+    entry::provide(providers);
     loops::provide(providers);
+    liveness::provide(providers);
+    intrinsicck::provide(providers);
 }
