@@ -1,9 +1,8 @@
-use std::fmt;
-use std::io;
 use std::path::PathBuf;
 
 use crate::externalfiles::ExternalHtml;
-use crate::html::render::SlashChecker;
+use crate::html::render::ensure_trailing_slash;
+use crate::html::format::{Buffer, Print};
 
 #[derive(Clone)]
 pub struct Layout {
@@ -11,6 +10,12 @@ pub struct Layout {
     pub favicon: String,
     pub external_html: ExternalHtml,
     pub krate: String,
+    /// The given user css file which allow to customize the generated
+    /// documentation theme.
+    pub css_file_extension: Option<PathBuf>,
+    /// If false, the `select` element to have search filtering by crates on rendered docs
+    /// won't be generated.
+    pub generate_search_filter: bool,
 }
 
 pub struct Page<'a> {
@@ -25,19 +30,15 @@ pub struct Page<'a> {
     pub static_extra_scripts: &'a [&'a str],
 }
 
-pub fn render<T: fmt::Display, S: fmt::Display>(
-    dst: &mut dyn io::Write,
+pub fn render<T: Print, S: Print>(
     layout: &Layout,
     page: &Page<'_>,
-    sidebar: &S,
-    t: &T,
-    css_file_extension: bool,
+    sidebar: S,
+    t: T,
     themes: &[PathBuf],
-    generate_search_filter: bool,
-) -> io::Result<()> {
+) -> String {
     let static_root_path = page.static_root_path.unwrap_or(page.root_path);
-    write!(dst,
-"<!DOCTYPE html>\
+    format!("<!DOCTYPE html>\
 <html lang=\"en\">\
 <head>\
     <meta charset=\"utf-8\">\
@@ -105,66 +106,19 @@ pub fn render<T: fmt::Display, S: fmt::Display>(
     <section id=\"main\" class=\"content\">{content}</section>\
     <section id=\"search\" class=\"content hidden\"></section>\
     <section class=\"footer\"></section>\
-    <aside id=\"help\" class=\"hidden\">\
-        <div>\
-            <h1 class=\"hidden\">Help</h1>\
-            <div class=\"shortcuts\">\
-                <h2>Keyboard Shortcuts</h2>\
-                <dl>\
-                    <dt><kbd>?</kbd></dt>\
-                    <dd>Show this help dialog</dd>\
-                    <dt><kbd>S</kbd></dt>\
-                    <dd>Focus the search field</dd>\
-                    <dt><kbd>↑</kbd></dt>\
-                    <dd>Move up in search results</dd>\
-                    <dt><kbd>↓</kbd></dt>\
-                    <dd>Move down in search results</dd>\
-                    <dt><kbd>↹</kbd></dt>\
-                    <dd>Switch tab</dd>\
-                    <dt><kbd>&#9166;</kbd></dt>\
-                    <dd>Go to active search result</dd>\
-                    <dt><kbd>+</kbd></dt>\
-                    <dd>Expand all sections</dd>\
-                    <dt><kbd>-</kbd></dt>\
-                    <dd>Collapse all sections</dd>\
-                </dl>\
-            </div>\
-            <div class=\"infos\">\
-                <h2>Search Tricks</h2>\
-                <p>\
-                    Prefix searches with a type followed by a colon (e.g., \
-                    <code>fn:</code>) to restrict the search to a given type.\
-                </p>\
-                <p>\
-                    Accepted types are: <code>fn</code>, <code>mod</code>, \
-                    <code>struct</code>, <code>enum</code>, \
-                    <code>trait</code>, <code>type</code>, <code>macro</code>, \
-                    and <code>const</code>.\
-                </p>\
-                <p>\
-                    Search functions by type signature (e.g., \
-                    <code>vec -> usize</code> or <code>* -> vec</code>)\
-                </p>\
-                <p>\
-                    Search multiple things at once by splitting your query with comma (e.g., \
-                    <code>str,u8</code> or <code>String,struct:Vec,test</code>)\
-                </p>\
-            </div>\
-        </div>\
-    </aside>\
     {after_content}\
     <script>\
         window.rootPath = \"{root_path}\";\
         window.currentCrate = \"{krate}\";\
     </script>\
-    <script src=\"{root_path}aliases.js\"></script>\
+    <script src=\"{root_path}aliases{suffix}.js\"></script>\
     <script src=\"{static_root_path}main{suffix}.js\"></script>\
     {static_extra_scripts}\
     {extra_scripts}\
-    <script defer src=\"{root_path}search-index.js\"></script>\
+    <script defer src=\"{root_path}search-index{suffix}.js\"></script>\
 </body>\
 </html>",
-    css_extension = if css_file_extension {
+    css_extension = if layout.css_file_extension.is_some() {
         format!("<link rel=\"stylesheet\" \
                        type=\"text/css\" \
                        href=\"{static_root_path}theme{suffix}.css\">",
@@ -173,23 +127,23 @@ pub fn render<T: fmt::Display, S: fmt::Display>(
     } else {
         String::new()
     },
-    content   = *t,
+    content   = Buffer::html().to_display(t),
     static_root_path = static_root_path,
     root_path = page.root_path,
     css_class = page.css_class,
     logo      = {
         let p = format!("{}{}", page.root_path, layout.krate);
-        let p = SlashChecker(&p);
+        let p = ensure_trailing_slash(&p);
         if layout.logo.is_empty() {
             format!("<a href='{path}index.html'>\
-                     <img src='{static_root_path}rust-logo{suffix}.png' \
-                          alt='logo' width='100'></a>",
+                     <div class='logo-container'>\
+                     <img src='{static_root_path}rust-logo{suffix}.png' alt='logo'></div></a>",
                     path=p,
                     static_root_path=static_root_path,
                     suffix=page.resource_suffix)
         } else {
             format!("<a href='{}index.html'>\
-                     <img src='{}' alt='logo' width='100'></a>",
+                     <div class='logo-container'><img src='{}' alt='logo'></div></a>",
                     p,
                     layout.logo)
         }
@@ -207,7 +161,7 @@ pub fn render<T: fmt::Display, S: fmt::Display>(
     in_header = layout.external_html.in_header,
     before_content = layout.external_html.before_content,
     after_content = layout.external_html.after_content,
-    sidebar   = *sidebar,
+    sidebar   = Buffer::html().to_display(sidebar),
     krate     = layout.krate,
     themes = themes.iter()
                    .filter_map(|t| t.file_stem())
@@ -228,7 +182,7 @@ pub fn render<T: fmt::Display, S: fmt::Display>(
                 root_path=page.root_path,
                 extra_script=e)
     }).collect::<String>(),
-    filter_crates=if generate_search_filter {
+    filter_crates=if layout.generate_search_filter {
         "<select id=\"crate-search\">\
             <option value=\"All crates\">All crates</option>\
         </select>"
@@ -238,9 +192,9 @@ pub fn render<T: fmt::Display, S: fmt::Display>(
     )
 }
 
-pub fn redirect(dst: &mut dyn io::Write, url: &str) -> io::Result<()> {
+pub fn redirect(url: &str) -> String {
     // <script> triggers a redirect before refresh, so this is fine.
-    write!(dst,
+    format!(
 r##"<!DOCTYPE html>
 <html lang="en">
 <head>
