@@ -1,11 +1,11 @@
-// compile-pass
+// build-pass (FIXME(62277): could be check-pass?)
 // pretty-expanded FIXME #23616
 
 pub trait Promisable: Send + Sync {}
 impl<T: Send + Sync> Promisable for T {}
 
 pub fn propagate<'a, T, E, F, G>(mut action: F)
-    -> Box<FnMut(Result<T, E>) -> Result<T, E> + 'a>
+    -> Box<dyn FnMut(Result<T, E>) -> Result<T, E> + 'a>
     where
         T: Promisable + Clone + 'a,
         E: Promisable + Clone + 'a,
