@@ -1,4 +1,4 @@
-type FuncType<'f> = Fn(&isize) -> isize + 'f;
+type FuncType<'f> = dyn Fn(&isize) -> isize + 'f;
 
 fn ho_func(f: Option<FuncType>) {
     //~^ ERROR the size for values of type
