@@ -1,10 +1,10 @@
-// compile-pass
+// build-pass (FIXME(62277): could be check-pass?)
 #![allow(unused_variables)]
 use std::collections::HashMap;
 use std::collections::hash_map::Entry::Vacant;
 
 pub fn foo() {
-    type F = Box<Fn(&()) + 'static>;
+    type F = Box<dyn Fn(&()) + 'static>;
     let mut map: HashMap<(), F> = HashMap::new();
     let x: &mut F = match map.entry(()) {
         Vacant(_) => unimplemented!(),
