@@ -2,9 +2,7 @@
 // implied bounds was causing outlives relations that were not
 // properly handled.
 //
-// compile-pass
-
-#![feature(nll)]
+// build-pass (FIXME(62277): could be check-pass?)
 
 fn main() {}
 
