@@ -1,5 +1,4 @@
-// compile-pass
-#![feature(nll)]
+// build-pass (FIXME(62277): could be check-pass?)
 
 // This test is reduced from a scenario pnkfelix encountered while
 // bootstrapping the compiler.
