@@ -15,7 +15,7 @@ fn f(_c: char) {}
 fn main() {
     match A::B(1, 2) {
         A::B(_, _, _) => (), //~ ERROR this pattern has 3 fields, but
-        A::D(_) => (),       //~ ERROR expected tuple struct/variant, found unit variant `A::D`
+        A::D(_) => (), //~ ERROR expected tuple struct or tuple variant, found unit variant `A::D`
         _ => ()
     }
     match 'c' {
